from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict, List, Optional

from .program import Program


@dataclass
class Programs:
    data: List[Program]
    next_page: Optional[int] = None

    def to_dict(self) -> Dict[str, Any]:
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)
        next_page = self.next_page

        return {"data": data, "nextPage": next_page}

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> Programs:
        data = []
        for data_item_data in d["data"]:
            data_item = Program.from_dict(data_item_data)
            data.append(data_item)
        next_page = d.get("nextPage")

        return Programs(data=data, next_page=next_page)



from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from typing import Any, Dict, List

from .program_scope import ProgramScope


@dataclass
class Program:
    program_url: str
    program_name: str
    platform: str
    rewards: List[str]
    types: List[str]
    in_scope: List[ProgramScope]
    out_scope: List[ProgramScope]
    bounty_created_at: datetime
    created_at: datetime
    updated_at: datetime
    slug: str
    live: bool
    minimum_bounty: int
    maximum_bounty: int
    average_bounty: int

    def to_dict(self) -> Dict[str, Any]:
        in_scope = []
        for in_scope_item_data in self.in_scope:
            in_scope_item = in_scope_item_data.to_dict()

            in_scope.append(in_scope_item)

        out_scope = []
        for out_scope_item_data in self.out_scope:
            out_scope_item = out_scope_item_data.to_dict()

            out_scope.append(out_scope_item)

        return {
            "programUrl": self.program_url,
            "programName": self.program_name,
            "platform": self.platform,
            "rewards": self.rewards,
            "types": self.types,
            "in_scope": in_scope,
            "out_scope": out_scope,
            "bountyCreatedAt": self.bounty_created_at.isoformat(),
            "createdAt": self.created_at.isoformat(),
            "updatedAt": self.updated_at.isoformat(),
            "slug": self.slug,
            "live": self.live,
            "minimumBounty": self.minimum_bounty,
            "maximumBounty": self.maximum_bounty,
            "averageBounty": self.average_bounty,
        }

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> Program:
        in_scope = []
        for in_scope_item_data in d["inScope"]:
            in_scope_item = ProgramScope.from_dict(in_scope_item_data)
            in_scope.append(in_scope_item)

        out_scope = []
        for out_scope_item_data in d["outScope"]:
            out_scope_item = ProgramScope.from_dict(out_scope_item_data)
            out_scope.append(out_scope_item)

        return Program(
            program_url=d["programUrl"],
            program_name=d["programName"],
            platform=d["platform"],
            rewards=d["rewards"],
            types=d["types"],
            in_scope=in_scope,
            out_scope=out_scope,
            bounty_created_at=datetime.strptime(
                d["bountyCreatedAt"], "%Y-%m-%dT%H:%M:%SZ"
            ),
            created_at=datetime.strptime(d["createdAt"], "%Y-%m-%dT%H:%M:%SZ"),
            updated_at=datetime.strptime(d["updatedAt"], "%Y-%m-%dT%H:%M:%SZ"),
            slug=d["slug"],
            live=d["live"],
            minimum_bounty=d["minimumBounty"],
            maximum_bounty=d["maximumBounty"],
            average_bounty=d["averageBounty"],
        )



from typing import Any, Dict, List, Optional, Union, cast
from datetime import datetime

import httpx

from ..client import Client
from ..errors import ApiResponseError
from ..models.http_validation_error import HTTPValidationError
from ..models.programs import Programs
from ..models.program import Program


def get_programs(
    *,
    client: Client,
    name: Optional[str] = None,
    types: Optional[List[str]] = None,
    platforms: Optional[List[str]] = None,
    exclude_platforms: Optional[List[str]] = None,
    rewards: Optional[List[str]] = None,
    page: Optional[int] = None,
    created_since: Optional[datetime] = None,
) -> Union[Programs, HTTPValidationError]:
    url = "{}/v0b/programs".format(client.base_url)

    params: Dict[str, Any] = {}
    if name is not None:
        params["name"] = name
    if types is not None:
        params["types"] = types
    if platforms is not None:
        params["platforms"] = platforms
    if exclude_platforms is not None:
        params["excludePlatforms"] = exclude_platforms
    if rewards is not None:
        params["rewards"] = rewards
    if page is not None:
        params["page"] = page
    if created_since is not None:
        params["createdSince"] = created_since.strftime("%Y-%m-%dT%H:%M:%SZ")

    response = httpx.get(url=url, headers=client.get_headers(), params=params)

    if response.status_code == 200:
        return Programs.from_dict(cast(Dict[str, Any], response.json()))
    if response.status_code == 422:
        return HTTPValidationError.from_dict(cast(Dict[str, Any], response.json()))
    if response.status_code == 401:
        return HTTPValidationError.from_dict(cast(Dict[str, Any], response.json()))
    else:
        raise ApiResponseError(response=response)


def get_program(*, client: Client, slug: str) -> Union[Program, HTTPValidationError]:
    url = "{}/v0b/programs/{slug}".format(client.base_url, slug=slug)
    response = httpx.get(url=url, headers=client.get_headers())

    if response.status_code == 200:
        return Program.from_dict(cast(Dict[str, Any], response.json()))
    if response.status_code == 422:
        return HTTPValidationError.from_dict(cast(Dict[str, Any], response.json()))
    if response.status_code == 401:
        return HTTPValidationError.from_dict(cast(Dict[str, Any], response.json()))
    else:
        raise ApiResponseError(response=response)



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict


@dataclass
class ProgramScope:
    type: str
    value: str

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        value = self.value

        return {"type": type, "value": value}

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> ProgramScope:
        type = d["type"]
        value = d["value"]

        return ProgramScope(type=type, value=value)



from typing import Optional, List, Generator
from pydantic import HttpUrl
from datetime import datetime

from .client import AuthenticatedClient
from .api.programs import get_programs, get_program
from .models import Program, HTTPValidationError


class APIException(Exception):
    pass


class BugBountyRecon:
    def __init__(
        self, token: str, *, base_url: HttpUrl = "https://api.bugbountyrecon.com"
    ):
        self.client = AuthenticatedClient(base_url=base_url, token=token)

    def _paginate(self, api_function, **kwargs):
        page = 0
        while page is not None:
            response = api_function(client=self.client, page=page, **kwargs)
            if isinstance(response, HTTPValidationError):
                raise APIException(response)
            for target in response.data:
                yield target
            page = response.next_page

    def program(self, slug: str) -> Program:
        return get_program(client=self.client, slug=slug)

    def programs(
        self,
        *,
        name: Optional[str] = None,
        types: Optional[List[str]] = None,
        platforms: Optional[List[str]] = None,
        exclude_platforms: Optional[List[str]] = None,
        rewards: Optional[List[str]] = None,
        created_since: Optional[datetime] = None,
    ) -> Generator[Program, None, None]:
        for program in self._paginate(
            get_programs,
            name=name,
            types=types,
            platforms=platforms,
            exclude_platforms=exclude_platforms,
            rewards=rewards,
            created_since=created_since,
        ):
            yield program



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict, List, Optional

from .validation_error import ValidationError


@dataclass
class HTTPValidationError:
    detail: Optional[List[ValidationError]] = None

    def to_dict(self) -> Dict[str, Any]:
        if self.detail is None:
            detail = None
        else:
            detail = []
            for detail_item_data in self.detail:
                detail_item = detail_item_data.to_dict()

                detail.append(detail_item)

        return {"detail": detail}

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> HTTPValidationError:
        detail = []
        for detail_item_data in d.get("detail") or []:
            detail_item = ValidationError.from_dict(detail_item_data)

            detail.append(detail_item)

        return HTTPValidationError(detail=detail)



""" Contains some shared types for properties """
from dataclasses import dataclass
from typing import BinaryIO, Optional, TextIO, Tuple, Union


@dataclass
class File:
    """ Contains information for file uploads """

    payload: Union[BinaryIO, TextIO]
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> Tuple[Optional[str], Union[BinaryIO, TextIO], Optional[str]]:
        return self.file_name, self.payload, self.mime_type


__all__ = ["File"]



# flake8: noqa
""" Contains all the data models used in inputs/outputs """

from .http_validation_error import HTTPValidationError
from .programs import Programs
from .program import Program
from .program_scope import ProgramScope
from .types import *
from .validation_error import ValidationError



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bbrecon', 'bbrecon.api', 'bbrecon.models']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.13.3,<0.14.0',
 'pydantic>=1.6.1,<2.0.0',
 'tabulate>=0.8.7,<0.9.0',
 'typer[all]>=0.3.1,<0.4.0']

entry_points = \
{'console_scripts': ['bbrecon = bbrecon.cli:main']}

setup_kwargs = {
    'name': 'bbrecon',
    'version': '0.0.0.1',
    'description': 'A client library and CLI for accessing the Bug Bounty Recon API',
    'long_description': '# bbrecon\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict, List


@dataclass
class ValidationError:
    loc: List[str]
    msg: str
    type: str

    def to_dict(self) -> Dict[str, Any]:
        loc = self.loc
        msg = self.msg
        type = self.type

        return {"loc": loc, "msg": msg, "type": type}

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> ValidationError:
        loc = d["loc"]
        msg = d["msg"]
        type = d["type"]

        return ValidationError(loc=loc, msg=msg, type=type)
